package main

import (
	"fmt"

	"github.com/beego/beego/v2/client/orm"
	"github.com/beego/beego/v2/core/logs"
	_ "github.com/go-sql-driver/mysql"
)

type City struct {
	ID   int `orm:"column(id);pk;auto"` // 第一个int类型的值会自动定义为主键，默认转为驼峰式 i_d
	Name string
}

// TableName 指定数据库表名称
func (c *City) TableName() string {
	return "city"
}

func main() {
	// debug 模式
	orm.Debug = true
	// db
	orm.RegisterDriver("mysql", orm.DRMySQL)
	// 可以通过 aliasName 指定多个数据库
	orm.RegisterDataBase("default", "mysql", "root:root@tcp(100.80.0.128:3306)/golangdev?charset=utf8")
	orm.SetMaxIdleConns("default", 10)
	orm.SetMaxOpenConns("default", 5)

	//orm.RegisterModel(&City{})
	orm.RegisterModel(new(City))

	/* 注册orm 命令
	   $ go run main.go orm
	   	orm command usage:

	   	    syncdb     - auto create tables
	   	    sqlall     - print sql of create tables
	   	    help       - print this help
	*/
	//orm.RunCommand()

	// 同步数据库
	orm.RunSyncdb("default", false, true)

	// 数据操作
	// 插入
	c := &City{
		//ID: 1, // ID 自增，不需要指定
		Name: "shanghai",
	}
	logs.Info(c)
	ormer := orm.NewOrm()
	//ormer := orm.NewOrmUsingDB("default")
	ormer.Insert(c)
	logs.Info(c) // 自动填充 id

	cs := make([]*City, 5, 5)
	for i := 0; i < 5; i++ {
		cs[i] = &City{Name: fmt.Sprintf("c_%d", i)}
	}
	ormer.InsertMulti(2, cs)

	// 查询
	cr := &City{ID: 1}
	err := ormer.Read(cr)
	if err != nil {
		logs.Info(err)
	}
	logs.Info(cr)

	cr = &City{Name: "shanghai"}
	err = ormer.Read(cr, "Name")
	if err != nil {
		logs.Info(err)
	}
	logs.Info(cr)

	// 读，不存在创建
	logs.Info(ormer.ReadOrCreate(cr, "Name"))

	// 查询所有
	//ormer.QueryTable("city")
	queryset := ormer.QueryTable(&City{})
	logs.Info(queryset)
	logs.Info(queryset.Count())

	var cs1 []*City
	queryset.All(&cs1)
	logs.Info(cs1)

	/*
		= exact iexact
		> gt
		< lt
		>= gte
		<= lte
		in in
		like contains startswith endswith icontains istartswith iendswith
	*/
	var cs2 []*City
	queryset.Filter("name", "shanghai").All(&cs2)
	logs.Info("cs2", cs2)
	var cs3 []*City
	queryset.Filter("id__gt", 5).All(&cs3)
	logs.Info("cs3", cs3)

	logs.Info(queryset.Filter("id__in", []int{1, 2, 3}).Count())
	logs.Info(queryset.Filter("id__gt", 5).Filter("id__lt", 8).Count())

	// 排查
	logs.Info(queryset.Exclude("id__gt", 20).Count())

	// 分页
	var cs4 []*City
	logs.Info(queryset.Filter("id__gt", 20).Limit(10).Offset(10).All(&cs4))
	logs.Info("cs4", cs4)

	//queryset.OrderBy("Name", "ID") // asc
	//queryset.OrderBy("-Name") // desc
	//queryset.One(&c)

	// 复杂查询
	//cond := orm.NewCondition()
	//condor := cond.Or("id__gt", 20).Or("id__lt", 20)
	//cond := cond.And("name__contains", "hai").AndCond(condor)
	//queryset.SetCond(cond)

	// 批量更新
	//queryset.Update(orm.Params{"name": "shanghai"})
	//queryset.Update(orm.Params{"name": "shanghai", "id": orm.ColValue(orm.ColAdd, 1)})

	//// 更新
	//cr.Name = "muodu"
	//ormer.Update(cr)
	//ormer.Update(cr, "Name")
	//
	//// 删除
	//ormer.Delete(&City{ID: 2})
	// 批量删除
	//queryset.Filter("id__gt", 20).Delete()

	// 原始 SQL
	// 原始的db
	//db, err := orm.GetDB("default")
	//db.Ping()
	//db.Exec()
	//db.Query()
	//db.QueryRow()

	//rawseter := ormer.Raw("update/create/delete")
	//rawseter.Exec()

	//rawseter := ormer.Raw("select * from city")
	//var c5 []*City
	//rawseter.QueryRows(&c5)

	//rawseter := ormer.Raw("select name, count(1) from city group by name")
	//var result []orm.Params
	//rawseter.Values(&result)
	//logs.Info(result)
	//
	//var resultList []orm.ParamsList
	//rawseter.ValuesList(&resultList)
	//logs.Info(resultList)

	// 事务
	//ormer.Begin()
	//...
}
